Package: nlmixr2plot
Title: Nonlinear Mixed Effects Models in Population PK/PD, Plot
        Functions
Version: 5.0.2
Authors@R: c(
    person("Matthew", "Fidler", email="matthew.fidler@gmail.com", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0001-8538-6691")),
    person("Bill", "Denney", email="wdenney@humanpredictions.com", role = "ctb",
           comment = c(ORCID = "0000-0002-5759-428X")),
    person("Wenping", "Wang", email="wwang8198@gmail.com", role = "aut"),
    person("Vipul", "Mann", email="vm2583@columbia.edu", role = "aut")
  )
Description: Fit and compare nonlinear mixed-effects models in
    differential equations with flexible dosing information commonly seen
    in pharmacokinetics and pharmacodynamics (Almquist, Leander, and
    Jirstrand 2015 <doi:10.1007/s10928-015-9409-1>). Differential equation
    solving is by compiled C code provided in the 'rxode2' package (Wang,
    Hallow, and James 2015 <doi:10.1002/psp4.12052>). This package is for
    'ggplot2' plotting methods for 'nlmixr2' objects.
License: GPL (>= 3)
URL: https://github.com/nlmixr2/nlmixr2plot,
        https://nlmixr2.github.io/nlmixr2plot/
BugReports: https://github.com/nlmixr2/nlmixr2plot/issues/
Imports: ggplot2 (>= 3.4.0), nlmixr2est, nlmixr2extra (>= 5.0.0),
        rxode2, tidyr, utils, xgxr
Suggests: testthat (>= 3.0.0), dplyr, withr, vpc, tidyvpc, nlmixr2data
Config/testthat/edition: 3
Encoding: UTF-8
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-06-10 18:12:15 UTC; matt-fidler
Author: Matthew Fidler [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-8538-6691>),
  Bill Denney [ctb] (ORCID: <https://orcid.org/0000-0002-5759-428X>),
  Wenping Wang [aut],
  Vipul Mann [aut]
Maintainer: Matthew Fidler <matthew.fidler@gmail.com>
Depends: R (>= 4.1.0)
Repository: CRAN
Date/Publication: 2026-06-11 05:10:08 UTC
Built: R 4.5.2; ; 2026-06-11 09:11:57 UTC; unix
