# Generated by roxygen2: do not edit by hand

S3method(Ops,nlmixr2PlotList)
S3method(chooseOpsMethod,nlmixr2PlotList)
S3method(plot,nlmixr2AugPred)
S3method(plot,nlmixr2FitCore)
S3method(plot,nlmixr2FitCoreSilent)
S3method(plot,nlmixr2FitData)
S3method(plot,nlmixr2PlotList)
S3method(print,nlmixr2PlotList)
S3method(traceplot,nlmixr2FitCore)
S3method(traceplot,nlmixr2FitCoreSilent)
export(geom_amt)
export(geom_cens)
export(stat_amt)
export(stat_cens)
export(traceplot)
export(vpcCens)
export(vpcCensTad)
export(vpcPlot)
export(vpcPlotTad)
importFrom(ggplot2,.data)
importFrom(rxode2,geom_amt)
importFrom(rxode2,geom_cens)
importFrom(rxode2,stat_amt)
importFrom(rxode2,stat_cens)
