% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reexports.R
\docType{import}
\name{reexports}
\alias{reexports}
\alias{geom_cens}
\alias{geom_amt}
\alias{stat_cens}
\alias{stat_amt}
\title{Objects exported from other packages}
\keyword{internal}
\description{
These objects are imported from other packages. Follow the links
below to see their documentation.

\describe{
  \item{rxode2}{\code{\link[rxode2:geom_amt]{geom_amt()}}, \code{\link[rxode2:geom_cens]{geom_cens()}}, \code{\link[rxode2:stat_amt]{stat_amt()}}, \code{\link[rxode2:stat_cens]{stat_cens()}}}
}}

